/*
 * Copyright 2012 Smallcrew, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package fr.smallcrew.rest.converter;

import org.codehaus.jackson.map.DeserializationConfig;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.map.ObjectWriter;
import org.codehaus.jackson.map.annotate.JsonSerialize;
import org.codehaus.jackson.map.type.TypeFactory;
import org.springframework.http.HttpInputMessage;
import org.springframework.http.HttpOutputMessage;
import org.springframework.http.MediaType;
import org.springframework.http.converter.AbstractHttpMessageConverter;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.http.converter.HttpMessageNotWritableException;

import java.io.IOException;
import java.util.Set;

public class JsonHttpMessageConverter extends AbstractHttpMessageConverter<Object> implements RestConverter {
  private final ObjectMapper objectMapper;
  private boolean usePrettyPrinter;

  public JsonHttpMessageConverter() {
    super(MediaType.APPLICATION_JSON);
    objectMapper = new ObjectMapper();
    objectMapper.setSerializationConfig(objectMapper.getSerializationConfig()
      .withSerializationInclusion(JsonSerialize.Inclusion.NON_NULL)
      .withSerializationInclusion(JsonSerialize.Inclusion.NON_EMPTY));
    objectMapper.configure(DeserializationConfig.Feature.FAIL_ON_UNKNOWN_PROPERTIES, false);
    usePrettyPrinter = true;
  }

  public void setUsePrettyPrinter(boolean usePrettyPrinter) {
    this.usePrettyPrinter = usePrettyPrinter;
  }

  @Override
  public boolean canRead(Class<?> clazz, MediaType mediaType) {
    return canRead(mediaType);
  }

  @Override
  public boolean canWrite(Class<?> clazz, MediaType mediaType) {
    return canWrite(mediaType);
  }

  @Override
  public boolean supports(Class<?> clazz) {
    return true;
  }

  @Override
  public Object readInternal(Class<?> clazz, HttpInputMessage inputMessage) throws IOException, HttpMessageNotReadableException {
    return objectMapper.readValue(inputMessage.getBody(), TypeFactory.fromCanonical(clazz.getCanonicalName()));
  }

  @Override
  public void writeInternal(Object o, HttpOutputMessage outputMessage) throws IOException, HttpMessageNotWritableException {
    getObjectWriter().writeValue(outputMessage.getBody(), o);
  }

  private ObjectWriter getObjectWriter() {
    if (usePrettyPrinter) {
      return objectMapper.writerWithDefaultPrettyPrinter();
    }
    return objectMapper.writer();
  }

  @Override
  public void addPackage(Set<String> packageName) {

  }

  @Override
  public void addClasses(Set<String> className) {

  }

  @Override
  public void consolidate() {

  }
}

